@@ -156,6 +156,7 @@ enum VKD3D_SHADER_INSTRUCTION_HANDLER
     VKD3DSIH_ENDREP,
     VKD3DSIH_ENDSWITCH,
     VKD3DSIH_EQ,
+    VKD3DSIH_EVAL_SAMPLE_INDEX,
     VKD3DSIH_EXP,
     VKD3DSIH_EXPP,
     VKD3DSIH_F16TOF32,
@@ -352,6 +353,9 @@ enum vkd3d_shader_register_type
     VKD3DSPR_COVERAGE,
     VKD3DSPR_SAMPLEMASK,
     VKD3DSPR_GSINSTID,
+    VKD3DSPR_DEPTHOUTGE,
+    VKD3DSPR_DEPTHOUTLE,
+    VKD3DSPR_RASTERIZER,
 
     VKD3DSPR_INVALID = ~0u,
 };
@@ -390,8 +394,6 @@ enum vkd3d_immconst_type
     VKD3D_IMMCONST_VEC4,
 };
 
-#define VKD3DSP_NOSWIZZLE (0u | (1u << 2) | (2u << 4) | (3u << 6))
-
 enum vkd3d_shader_src_modifier
 {
     VKD3DSPSM_NONE = 0,
@@ -438,6 +440,7 @@ enum vkd3d_shader_interpolation_mode
 enum vkd3d_shader_global_flags
 {
     VKD3DSGF_REFACTORING_ALLOWED               = 0x1,
+    VKD3DSGF_FORCE_EARLY_DEPTH_STENCIL         = 0x4,
     VKD3DSGF_ENABLE_RAW_AND_STRUCTURED_BUFFERS = 0x8,
 };
 
@@ -476,12 +479,23 @@ enum vkd3d_tessellator_partitioning
     VKD3D_TESSELLATOR_PARTITIONING_FRACTIONAL_EVEN = 4,
 };
 
-#define VKD3DSI_INDEXED_DYNAMIC   0x4
-#define VKD3DSI_RESINFO_RCP_FLOAT 0x1
-#define VKD3DSI_RESINFO_UINT      0x2
-#define VKD3DSI_SAMPLE_INFO_UINT  0x1
+
+#define VKD3DSI_NONE                    0x0
+#define VKD3DSI_TEXLD_PROJECT           0x1
+#define VKD3DSI_INDEXED_DYNAMIC         0x4
+#define VKD3DSI_RESINFO_RCP_FLOAT       0x1
+#define VKD3DSI_RESINFO_UINT            0x2
+#define VKD3DSI_SAMPLE_INFO_UINT        0x1
 #define VKD3DSI_SAMPLER_COMPARISON_MODE 0x1
 
+#define VKD3DSI_PRECISE_X         0x100
+#define VKD3DSI_PRECISE_Y         0x200
+#define VKD3DSI_PRECISE_Z         0x400
+#define VKD3DSI_PRECISE_W         0x800
+#define VKD3DSI_PRECISE_XYZW      (VKD3DSI_PRECISE_X | VKD3DSI_PRECISE_Y \
+                                  | VKD3DSI_PRECISE_Z | VKD3DSI_PRECISE_W)
+#define VKD3DSI_PRECISE_SHIFT     8
+
 enum vkd3d_shader_rel_op
 {
     VKD3D_SHADER_REL_OP_GT = 1,
@@ -562,7 +576,8 @@ struct vkd3d_shader_register
     enum vkd3d_immconst_type immconst_type;
     union
     {
-        DWORD immconst_data[VKD3D_VEC4_SIZE];
+        DWORD immconst_uint[VKD3D_VEC4_SIZE];
+        float immconst_float[VKD3D_VEC4_SIZE];
         unsigned fp_body_idx;
     } u;
 };
@@ -732,6 +747,8 @@ struct vkd3d_shader_instruction
     const struct vkd3d_shader_dst_param *dst;
     const struct vkd3d_shader_src_param *src;
     struct vkd3d_shader_texel_offset texel_offset;
+    bool coissue;
+    const struct vkd3d_shader_src_param *predicate;
     union
     {
         struct vkd3d_shader_semantic semantic;
@@ -761,6 +778,13 @@ static inline BOOL vkd3d_shader_instruction_has_texel_offset(const struct vkd3d_
     return ins->texel_offset.u || ins->texel_offset.v || ins->texel_offset.w;
 }
 
+static inline bool vkd3d_shader_register_is_input(const struct vkd3d_shader_register *reg)
+{
+    return reg->type == VKD3DSPR_INPUT || reg->type == VKD3DSPR_INCONTROLPOINT;
+}
+
+void vkd3d_shader_trace(void *data) DECLSPEC_HIDDEN;
+
 void *shader_sm4_init(const DWORD *byte_code, size_t byte_code_size,
         const struct vkd3d_shader_signature *output_signature) DECLSPEC_HIDDEN;
 void shader_sm4_free(void *data) DECLSPEC_HIDDEN;
@@ -782,6 +806,7 @@ struct vkd3d_dxbc_compiler;
 struct vkd3d_dxbc_compiler *vkd3d_dxbc_compiler_create(const struct vkd3d_shader_version *shader_version,
         const struct vkd3d_shader_desc *shader_desc, uint32_t compiler_options,
         const struct vkd3d_shader_interface *shader_interface,
+        const struct vkd3d_shader_compile_arguments *compile_args,
         const struct vkd3d_shader_scan_info *scan_info) DECLSPEC_HIDDEN;
 void vkd3d_dxbc_compiler_handle_instruction(struct vkd3d_dxbc_compiler *compiler,
         const struct vkd3d_shader_instruction *instruction) DECLSPEC_HIDDEN;
@@ -852,11 +877,16 @@ static inline unsigned int vkd3d_write_mask_component_count(DWORD write_mask)
     return count;
 }
 
+static inline unsigned int vkd3d_write_mask_from_component_count(unsigned int component_count)
+{
+    assert(component_count <= 4);
+    return (VKD3DSP_WRITEMASK_0 << component_count) - 1;
+}
+
 static inline unsigned int vkd3d_swizzle_get_component(DWORD swizzle,
         unsigned int idx)
 {
-    /* swizzle bits fields: wwzzyyxx */
-    return (swizzle >> (2 * idx)) & 0x3;
+    return (swizzle >> VKD3D_SWIZZLE_SHIFT(idx)) & VKD3D_SWIZZLE_MASK;
 }
 
 #define VKD3D_DXBC_MAX_SOURCE_COUNT 6
