@@ -24,55 +24,82 @@
 #define DEPTH_STENCIL (VK_IMAGE_ASPECT_DEPTH_BIT | VK_IMAGE_ASPECT_STENCIL_BIT)
 static const struct vkd3d_format vkd3d_formats[] =
 {
-    {DXGI_FORMAT_R32G32B32A32_FLOAT,  VK_FORMAT_R32G32B32A32_SFLOAT,      16, 1, 1,  1, COLOR},
-    {DXGI_FORMAT_R32G32B32A32_UINT,   VK_FORMAT_R32G32B32A32_UINT,        16, 1, 1,  1, COLOR},
-    {DXGI_FORMAT_R32G32B32A32_SINT,   VK_FORMAT_R32G32B32A32_SINT,        16, 1, 1,  1, COLOR},
-    {DXGI_FORMAT_R32G32B32_FLOAT,     VK_FORMAT_R32G32B32_SFLOAT,         12, 1, 1,  1, COLOR},
-    {DXGI_FORMAT_R32G32_FLOAT,        VK_FORMAT_R32G32_SFLOAT,            8,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R16G16B16A16_FLOAT,  VK_FORMAT_R16G16B16A16_SFLOAT,      8,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R16G16B16A16_UINT,   VK_FORMAT_R16G16B16A16_UINT,        8,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R10G10B10A2_UNORM,   VK_FORMAT_A2B10G10R10_UNORM_PACK32, 4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R11G11B10_FLOAT,     VK_FORMAT_B10G11R11_UFLOAT_PACK32,  4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R8G8B8A8_TYPELESS,   VK_FORMAT_R8G8B8A8_UNORM,           4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R8G8B8A8_UNORM,      VK_FORMAT_R8G8B8A8_UNORM,           4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R8G8B8A8_UNORM_SRGB, VK_FORMAT_R8G8B8A8_SRGB,            4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R16G16_FLOAT,        VK_FORMAT_R16G16_SFLOAT,            4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R16G16_UNORM,        VK_FORMAT_R16G16_UNORM,             4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R32_TYPELESS,        VK_FORMAT_R32_UINT,                 4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_D32_FLOAT,           VK_FORMAT_D32_SFLOAT,               4,  1, 1,  1, DEPTH},
-    {DXGI_FORMAT_R32_FLOAT,           VK_FORMAT_R32_SFLOAT,               4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R32_UINT,            VK_FORMAT_R32_UINT,                 4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R32_SINT,            VK_FORMAT_R32_SINT,                 4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R16_TYPELESS,        VK_FORMAT_R16_UINT,                 2,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R16_FLOAT,           VK_FORMAT_R16_SFLOAT,               2,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_D16_UNORM,           VK_FORMAT_D16_UNORM,                2,  1, 1,  1, DEPTH},
-    {DXGI_FORMAT_R16_UNORM,           VK_FORMAT_R16_UNORM,                2,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R8_UNORM,            VK_FORMAT_R8_UNORM,                 1,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R8_UINT,             VK_FORMAT_R8_UINT,                  1,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_R8_SNORM,            VK_FORMAT_R8_SNORM,                 1,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_A8_UNORM,            VK_FORMAT_R8_UNORM,                 1,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_B8G8R8A8_UNORM,      VK_FORMAT_B8G8R8A8_UNORM,           4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_B8G8R8A8_TYPELESS,   VK_FORMAT_B8G8R8A8_UNORM,           4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_B8G8R8A8_UNORM_SRGB, VK_FORMAT_B8G8R8A8_SRGB,            4,  1, 1,  1, COLOR},
-    {DXGI_FORMAT_BC1_TYPELESS,        VK_FORMAT_BC1_RGBA_UNORM_BLOCK,     1,  4, 4,  8, COLOR},
-    {DXGI_FORMAT_BC1_UNORM,           VK_FORMAT_BC1_RGBA_UNORM_BLOCK,     1,  4, 4,  8, COLOR},
-    {DXGI_FORMAT_BC1_UNORM_SRGB,      VK_FORMAT_BC1_RGBA_SRGB_BLOCK,      1,  4, 4,  8, COLOR},
-    {DXGI_FORMAT_BC2_TYPELESS,        VK_FORMAT_BC2_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC2_UNORM,           VK_FORMAT_BC2_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC2_UNORM_SRGB,      VK_FORMAT_BC2_SRGB_BLOCK,           1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC3_TYPELESS,        VK_FORMAT_BC3_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC3_UNORM,           VK_FORMAT_BC3_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC3_UNORM_SRGB,      VK_FORMAT_BC3_SRGB_BLOCK,           1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC4_TYPELESS,        VK_FORMAT_BC4_UNORM_BLOCK,          1,  4, 4,  8, COLOR},
-    {DXGI_FORMAT_BC4_UNORM,           VK_FORMAT_BC4_UNORM_BLOCK,          1,  4, 4,  8, COLOR},
-    {DXGI_FORMAT_BC4_SNORM,           VK_FORMAT_BC4_SNORM_BLOCK,          1,  4, 4,  8, COLOR},
-    {DXGI_FORMAT_BC5_TYPELESS,        VK_FORMAT_BC5_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC5_UNORM,           VK_FORMAT_BC5_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC5_SNORM,           VK_FORMAT_BC5_SNORM_BLOCK,          1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC6H_UF16,           VK_FORMAT_BC6H_UFLOAT_BLOCK,        1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC6H_SF16,           VK_FORMAT_BC6H_SFLOAT_BLOCK,        1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC7_UNORM,           VK_FORMAT_BC7_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
-    {DXGI_FORMAT_BC7_UNORM_SRGB,      VK_FORMAT_BC7_SRGB_BLOCK,           1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_R32G32B32A32_TYPELESS, VK_FORMAT_R32G32B32A32_SFLOAT,      16, 1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32B32A32_FLOAT,    VK_FORMAT_R32G32B32A32_SFLOAT,      16, 1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32B32A32_UINT,     VK_FORMAT_R32G32B32A32_UINT,        16, 1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32B32A32_SINT,     VK_FORMAT_R32G32B32A32_SINT,        16, 1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32B32_FLOAT,       VK_FORMAT_R32G32B32_SFLOAT,         12, 1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32B32_UINT,        VK_FORMAT_R32G32B32_UINT,           12, 1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32B32_SINT,        VK_FORMAT_R32G32B32_SINT,           12, 1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16B16A16_TYPELESS, VK_FORMAT_R16G16B16A16_SFLOAT,      8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16B16A16_FLOAT,    VK_FORMAT_R16G16B16A16_SFLOAT,      8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16B16A16_UNORM,    VK_FORMAT_R16G16B16A16_UNORM,       8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16B16A16_UINT,     VK_FORMAT_R16G16B16A16_UINT,        8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16B16A16_SNORM,    VK_FORMAT_R16G16B16A16_SNORM,       8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16B16A16_SINT,     VK_FORMAT_R16G16B16A16_SINT,        8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32_TYPELESS,       VK_FORMAT_R32G32_SFLOAT,            8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32_FLOAT,          VK_FORMAT_R32G32_SFLOAT,            8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32_UINT,           VK_FORMAT_R32G32_UINT,              8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32G32_SINT,           VK_FORMAT_R32G32_SINT,              8,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R10G10B10A2_UNORM,     VK_FORMAT_A2B10G10R10_UNORM_PACK32, 4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R11G11B10_FLOAT,       VK_FORMAT_B10G11R11_UFLOAT_PACK32,  4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8_TYPELESS,         VK_FORMAT_R8G8_UNORM,               2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8_UNORM,            VK_FORMAT_R8G8_UNORM,               2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8_UINT,             VK_FORMAT_R8G8_UINT,                2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8_SNORM,            VK_FORMAT_R8G8_SNORM,               2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8_SINT,             VK_FORMAT_R8G8_SINT,                2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8B8A8_TYPELESS,     VK_FORMAT_R8G8B8A8_UNORM,           4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8B8A8_UNORM,        VK_FORMAT_R8G8B8A8_UNORM,           4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8B8A8_UNORM_SRGB,   VK_FORMAT_R8G8B8A8_SRGB,            4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8B8A8_UINT,         VK_FORMAT_R8G8B8A8_UINT,            4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8B8A8_SNORM,        VK_FORMAT_R8G8B8A8_SNORM,           4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8G8B8A8_SINT,         VK_FORMAT_R8G8B8A8_SINT,            4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16_TYPELESS,       VK_FORMAT_R16G16_SFLOAT,            4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16_FLOAT,          VK_FORMAT_R16G16_SFLOAT,            4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16_UNORM,          VK_FORMAT_R16G16_UNORM,             4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16_UINT,           VK_FORMAT_R16G16_UINT,              4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16_SNORM,          VK_FORMAT_R16G16_SNORM,             4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16G16_SINT,           VK_FORMAT_R16G16_SINT,              4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32_TYPELESS,          VK_FORMAT_R32_UINT,                 4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_D32_FLOAT,             VK_FORMAT_D32_SFLOAT,               4,  1, 1,  1, DEPTH},
+    {DXGI_FORMAT_R32_FLOAT,             VK_FORMAT_R32_SFLOAT,               4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32_UINT,              VK_FORMAT_R32_UINT,                 4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R32_SINT,              VK_FORMAT_R32_SINT,                 4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16_TYPELESS,          VK_FORMAT_R16_UINT,                 2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16_FLOAT,             VK_FORMAT_R16_SFLOAT,               2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_D16_UNORM,             VK_FORMAT_D16_UNORM,                2,  1, 1,  1, DEPTH},
+    {DXGI_FORMAT_R16_UNORM,             VK_FORMAT_R16_UNORM,                2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16_UINT,              VK_FORMAT_R16_UINT,                 2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16_SNORM,             VK_FORMAT_R16_SNORM,                2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R16_SINT,              VK_FORMAT_R16_SINT,                 2,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8_TYPELESS,           VK_FORMAT_R8_UNORM,                 1,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8_UNORM,              VK_FORMAT_R8_UNORM,                 1,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8_UINT,               VK_FORMAT_R8_UINT,                  1,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8_SNORM,              VK_FORMAT_R8_SNORM,                 1,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_R8_SINT,               VK_FORMAT_R8_SINT,                  1,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_A8_UNORM,              VK_FORMAT_R8_UNORM,                 1,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_B8G8R8A8_UNORM,        VK_FORMAT_B8G8R8A8_UNORM,           4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_B8G8R8A8_TYPELESS,     VK_FORMAT_B8G8R8A8_UNORM,           4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_B8G8R8A8_UNORM_SRGB,   VK_FORMAT_B8G8R8A8_SRGB,            4,  1, 1,  1, COLOR},
+    {DXGI_FORMAT_BC1_TYPELESS,          VK_FORMAT_BC1_RGBA_UNORM_BLOCK,     1,  4, 4,  8, COLOR},
+    {DXGI_FORMAT_BC1_UNORM,             VK_FORMAT_BC1_RGBA_UNORM_BLOCK,     1,  4, 4,  8, COLOR},
+    {DXGI_FORMAT_BC1_UNORM_SRGB,        VK_FORMAT_BC1_RGBA_SRGB_BLOCK,      1,  4, 4,  8, COLOR},
+    {DXGI_FORMAT_BC2_TYPELESS,          VK_FORMAT_BC2_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC2_UNORM,             VK_FORMAT_BC2_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC2_UNORM_SRGB,        VK_FORMAT_BC2_SRGB_BLOCK,           1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC3_TYPELESS,          VK_FORMAT_BC3_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC3_UNORM,             VK_FORMAT_BC3_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC3_UNORM_SRGB,        VK_FORMAT_BC3_SRGB_BLOCK,           1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC4_TYPELESS,          VK_FORMAT_BC4_UNORM_BLOCK,          1,  4, 4,  8, COLOR},
+    {DXGI_FORMAT_BC4_UNORM,             VK_FORMAT_BC4_UNORM_BLOCK,          1,  4, 4,  8, COLOR},
+    {DXGI_FORMAT_BC4_SNORM,             VK_FORMAT_BC4_SNORM_BLOCK,          1,  4, 4,  8, COLOR},
+    {DXGI_FORMAT_BC5_TYPELESS,          VK_FORMAT_BC5_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC5_UNORM,             VK_FORMAT_BC5_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC5_SNORM,             VK_FORMAT_BC5_SNORM_BLOCK,          1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC6H_UF16,             VK_FORMAT_BC6H_UFLOAT_BLOCK,        1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC6H_SF16,             VK_FORMAT_BC6H_SFLOAT_BLOCK,        1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC7_UNORM,             VK_FORMAT_BC7_UNORM_BLOCK,          1,  4, 4, 16, COLOR},
+    {DXGI_FORMAT_BC7_UNORM_SRGB,        VK_FORMAT_BC7_SRGB_BLOCK,           1,  4, 4, 16, COLOR},
 };
 
 /* Each depth/stencil format is only compatible with itself in Vulkan. */
@@ -173,6 +200,24 @@ bool dxgi_format_is_typeless(DXGI_FORMAT dxgi_format)
     }
 }
 
+DXGI_FORMAT vkd3d_get_dxgi_format(VkFormat format)
+{
+    DXGI_FORMAT dxgi_format;
+    VkFormat vk_format;
+    unsigned int i;
+
+    for (i = 0; i < ARRAY_SIZE(vkd3d_formats); ++i)
+    {
+        vk_format = vkd3d_formats[i].vk_format;
+        dxgi_format = vkd3d_formats[i].dxgi_format;
+        if (vk_format == format && !dxgi_format_is_typeless(dxgi_format))
+            return dxgi_format;
+    }
+
+    FIXME("Unhandled Vulkan format %#x.\n", format);
+    return DXGI_FORMAT_UNKNOWN;
+}
+
 bool is_valid_feature_level(D3D_FEATURE_LEVEL feature_level)
 {
     static const D3D_FEATURE_LEVEL valid_feature_levels[] =
@@ -256,19 +301,20 @@ bool is_valid_resource_state(D3D12_RESOURCE_STATES state)
     return true;
 }
 
-HRESULT return_interface(IUnknown *iface, REFIID iface_riid,
-        REFIID requested_riid, void **object)
+HRESULT return_interface(void *iface, REFIID iface_iid,
+        REFIID requested_iid, void **object)
 {
+    IUnknown *unknown = iface;
     HRESULT hr;
 
-    if (IsEqualGUID(iface_riid, requested_riid))
+    if (IsEqualGUID(iface_iid, requested_iid))
     {
-        *object = iface;
+        *object = unknown;
         return S_OK;
     }
 
-    hr = IUnknown_QueryInterface(iface, requested_riid, object);
-    IUnknown_Release(iface);
+    hr = IUnknown_QueryInterface(unknown, requested_iid, object);
+    IUnknown_Release(unknown);
     return hr;
 }
 
@@ -341,8 +387,9 @@ HRESULT hresult_from_vk_result(VkResult vr)
     {
         case VK_SUCCESS:
             return S_OK;
-        case VK_ERROR_OUT_OF_HOST_MEMORY:
         case VK_ERROR_OUT_OF_DEVICE_MEMORY:
+            WARN("Out of device memory.\n");
+        case VK_ERROR_OUT_OF_HOST_MEMORY:
             return E_OUTOFMEMORY;
         default:
             FIXME("Unhandled VkResult %d.\n", vr);
