@@ -31,6 +31,17 @@ static void set_rect(RECT *rect, int left, int top, int right, int bottom)
     rect->bottom = bottom;
 }
 
+static inline void set_box(D3D12_BOX *box, unsigned int left, unsigned int top, unsigned int front,
+        unsigned int right, unsigned int bottom, unsigned int back)
+{
+    box->left = left;
+    box->top = top;
+    box->front = front;
+    box->right = right;
+    box->bottom = bottom;
+    box->back = back;
+}
+
 static void set_viewport(D3D12_VIEWPORT *vp, float x, float y,
         float width, float height, float min_depth, float max_depth)
 {
@@ -197,6 +208,7 @@ struct resource_readback
 {
     unsigned int width;
     unsigned int height;
+    unsigned int depth;
     ID3D12Resource *resource;
     unsigned int row_pitch;
     void *data;
@@ -219,18 +231,18 @@ static void get_texture_readback_with_command_list(ID3D12Resource *texture, unsi
     resource_desc = ID3D12Resource_GetDesc(texture);
     ok(resource_desc.Dimension != D3D12_RESOURCE_DIMENSION_BUFFER,
             "Resource %p is not texture.\n", texture);
-    ok(resource_desc.Dimension != D3D12_RESOURCE_DIMENSION_TEXTURE3D,
-            "Readback not implemented for 3D textures.\n");
 
     miplevel = sub_resource % resource_desc.MipLevels;
     rb->width = max(1, resource_desc.Width >> miplevel);
     rb->height = max(1, resource_desc.Height >> miplevel);
+    rb->depth = resource_desc.Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE3D
+            ? resource_desc.DepthOrArraySize : 1;
     rb->row_pitch = align(rb->width * format_size(resource_desc.Format), D3D12_TEXTURE_DATA_PITCH_ALIGNMENT);
     rb->data = NULL;
 
     format = resource_desc.Format;
 
-    rb->resource = create_readback_buffer(device, rb->row_pitch * rb->height);
+    rb->resource = create_readback_buffer(device, rb->row_pitch * rb->height * rb->depth);
 
     dst_location.pResource = rb->resource;
     dst_location.Type = D3D12_TEXTURE_COPY_TYPE_PLACED_FOOTPRINT;
@@ -238,7 +250,7 @@ static void get_texture_readback_with_command_list(ID3D12Resource *texture, unsi
     dst_location.PlacedFootprint.Footprint.Format = format;
     dst_location.PlacedFootprint.Footprint.Width = rb->width;
     dst_location.PlacedFootprint.Footprint.Height = rb->height;
-    dst_location.PlacedFootprint.Footprint.Depth = 1;
+    dst_location.PlacedFootprint.Footprint.Depth = rb->depth;
     dst_location.PlacedFootprint.Footprint.RowPitch = rb->row_pitch;
 
     src_location.pResource = texture;
@@ -259,15 +271,17 @@ static void get_texture_readback_with_command_list(ID3D12Resource *texture, unsi
     ok(SUCCEEDED(hr), "Failed to map readback buffer, hr %#x.\n", hr);
 }
 
-static void *get_readback_data(struct resource_readback *rb, unsigned int x, unsigned int y,
-        size_t element_size)
+static void *get_readback_data(struct resource_readback *rb,
+        unsigned int x, unsigned int y, unsigned int z, size_t element_size)
 {
-    return &((BYTE *)rb->data)[rb->row_pitch * y + x * element_size];
+    unsigned int slice_pitch = rb->row_pitch * rb->depth;
+    return &((BYTE *)rb->data)[slice_pitch * z + rb->row_pitch * y + x * element_size];
 }
 
-static unsigned int get_readback_uint(struct resource_readback *rb, unsigned int x, unsigned int y)
+static unsigned int get_readback_uint(struct resource_readback *rb,
+        unsigned int x, unsigned int y, unsigned int z)
 {
-    return *(unsigned int *)get_readback_data(rb, x, y, sizeof(unsigned int));
+    return *(unsigned int *)get_readback_data(rb, x, y, z, sizeof(unsigned int));
 }
 
 static void release_resource_readback(struct resource_readback *rb)
@@ -279,31 +293,36 @@ static void release_resource_readback(struct resource_readback *rb)
 
 #define check_readback_data_uint(a, b, c, d) check_readback_data_uint_(__LINE__, a, b, c, d)
 static void check_readback_data_uint_(unsigned int line, struct resource_readback *rb,
-        const RECT *rect, unsigned int expected, unsigned int max_diff)
+        const D3D12_BOX *box, unsigned int expected, unsigned int max_diff)
 {
-    RECT r = {0, 0, rb->width, rb->height};
-    unsigned int x = 0, y;
+    D3D12_BOX b = {0, 0, 0, rb->width, rb->height, rb->depth};
+    unsigned int x = 0, y = 0, z;
     bool all_match = true;
     unsigned int got = 0;
 
-    if (rect)
-        r = *rect;
+    if (box)
+        b = *box;
 
-    for (y = r.top; y < r.bottom; ++y)
+    for (z = b.front; z < b.back; ++z)
     {
-        for (x = r.left; x < r.right; ++x)
+        for (y = b.top; y < b.bottom; ++y)
         {
-            got = get_readback_uint(rb, x, y);
-            if (!compare_color(got, expected, max_diff))
+            for (x = b.left; x < b.right; ++x)
             {
-                all_match = false;
-                break;
+                got = get_readback_uint(rb, x, y, z);
+                if (!compare_color(got, expected, max_diff))
+                {
+                    all_match = false;
+                    break;
+                }
             }
+            if (!all_match)
+                break;
         }
         if (!all_match)
             break;
     }
-    ok_(line)(all_match, "Got 0x%08x, expected 0x%08x at (%u, %u).\n", got, expected, x, y);
+    ok_(line)(all_match, "Got 0x%08x, expected 0x%08x at (%u, %u, %u).\n", got, expected, x, y, z);
 }
 
 #define check_sub_resource_uint(a, b, c, d, e, f) check_sub_resource_uint_(__LINE__, a, b, c, d, e, f)
@@ -313,16 +332,16 @@ static void check_sub_resource_uint_(unsigned int line, ID3D12Resource *texture,
 {
     struct resource_readback rb;
 
-    get_texture_readback_with_command_list(texture, 0, &rb, queue, command_list);
+    get_texture_readback_with_command_list(texture, sub_resource_idx, &rb, queue, command_list);
     check_readback_data_uint_(line, &rb, NULL, expected, max_diff);
     release_resource_readback(&rb);
 }
 
-#define create_texture(a, b, c, d, e) create_default_texture_(__LINE__, a, b, c, d, 0, e)
-#define create_default_texture(a, b, c, d, e, f) create_default_texture_(__LINE__, a, b, c, d, e, f)
-static ID3D12Resource *create_default_texture_(unsigned int line, ID3D12Device *device,
-        unsigned int width, unsigned int height, DXGI_FORMAT format,
-        D3D12_RESOURCE_FLAGS flags, D3D12_RESOURCE_STATES initial_state)
+#define create_default_texture(a, b, c, d, e, f) create_default_texture2d_(__LINE__, a, b, c, 1, 1, d, e, f)
+#define create_default_texture2d(a, b, c, d, e, f, g, h) create_default_texture2d_(__LINE__, a, b, c, d, e, f, g, h)
+static ID3D12Resource *create_default_texture2d_(unsigned int line, ID3D12Device *device,
+        unsigned int width, unsigned int height, unsigned int array_size, unsigned int miplevel_count,
+        DXGI_FORMAT format, D3D12_RESOURCE_FLAGS flags, D3D12_RESOURCE_STATES initial_state)
 {
     D3D12_HEAP_PROPERTIES heap_properties;
     D3D12_RESOURCE_DESC resource_desc;
@@ -336,8 +355,8 @@ static ID3D12Resource *create_default_texture_(unsigned int line, ID3D12Device *
     resource_desc.Dimension = D3D12_RESOURCE_DIMENSION_TEXTURE2D;
     resource_desc.Width = width;
     resource_desc.Height = height;
-    resource_desc.DepthOrArraySize = 1;
-    resource_desc.MipLevels = 1;
+    resource_desc.DepthOrArraySize = array_size;
+    resource_desc.MipLevels = miplevel_count;
     resource_desc.Format = format;
     resource_desc.SampleDesc.Count = 1;
     resource_desc.Flags = flags;
@@ -492,7 +511,7 @@ static ID3D12PipelineState *create_pipeline_state_(unsigned int line, ID3D12Devi
 
 struct test_context_desc
 {
-    unsigned int rt_width, rt_height;
+    unsigned int rt_width, rt_height, rt_array_size;
     DXGI_FORMAT rt_format;
     unsigned int rt_descriptor_count;
     unsigned int root_signature_flags;
@@ -540,7 +559,7 @@ static void create_render_target_(unsigned int line, struct test_context *contex
     resource_desc.Alignment = 0;
     resource_desc.Width = desc && desc->rt_width ? desc->rt_width : 32;
     resource_desc.Height = desc && desc->rt_height ? desc->rt_height : 32;
-    resource_desc.DepthOrArraySize = 1;
+    resource_desc.DepthOrArraySize = desc && desc->rt_array_size ? desc->rt_array_size : 1;
     resource_desc.MipLevels = 1;
     resource_desc.Format = desc && desc->rt_format ? desc->rt_format : DXGI_FORMAT_R8G8B8A8_UNORM;
     resource_desc.SampleDesc.Count = 1;
